"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const debug_1 = tslib_1.__importDefault(require("debug"));
const kleur_1 = tslib_1.__importDefault(require("kleur"));
const path_1 = require("path");
const cli_1 = require("./cli");
const fs_1 = require("./fs");
const help = tslib_1.__importStar(require("./help"));
const options_1 = require("./options");
const prompt_1 = require("./prompt");
const subprocess_1 = require("./subprocess");
const template_1 = require("./template");
const debug = debug_1.default('@capacitor/create-plugin');
process.on('unhandledRejection', error => {
    process.stderr.write(`ERR: ${error}\n`);
    process.exit(1);
});
exports.run = async () => {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    if (process.argv.find(arg => ['-h', '-?', '--help'].includes(arg))) {
        help.run();
        process.exit();
    }
    const options = options_1.getOptions();
    debug('options from command-line: %O', options);
    if (Object.values(options).includes(undefined)) {
        if (cli_1.isTTY) {
            debug(`Missing/invalid options. Prompting for user input...`);
        }
        else {
            process.stderr.write(`ERR: Refusing to prompt for missing/invalid options in non-TTY environment.\n` +
                `See ${kleur_1.default.bold('--help')}. Run with ${kleur_1.default.bold('--verbose')} for more context.\n`);
            process.exit(1);
        }
    }
    const details = await prompt_1.gatherDetails(options);
    const dir = path_1.resolve(process.cwd(), details.dir);
    if (await fs_1.exists(dir)) {
        process.stderr.write(`ERR: Not overwriting existing directory: ${kleur_1.default.bold(details.dir)}`);
        process.exit(1);
    }
    await template_1.extractTemplate(dir, details);
    process.stdout.write('Installing dependencies. Please wait...\n');
    const opts = { cwd: details.dir, stdio: 'inherit' };
    try {
        await subprocess_1.run('npm', ['install', '--no-package-lock'], opts);
        try {
            await subprocess_1.run('npm', ['run', 'fmt'], opts);
        }
        catch (e) {
            process.stderr.write(`WARN: Could not format source files: ${(_b = (_a = e.message) !== null && _a !== void 0 ? _a : e.stack) !== null && _b !== void 0 ? _b : e}\n`);
        }
    }
    catch (e) {
        process.stderr.write(`WARN: Could not install dependencies: ${(_d = (_c = e.message) !== null && _c !== void 0 ? _c : e.stack) !== null && _d !== void 0 ? _d : e}\n`);
    }
    if (process.platform === 'darwin') {
        try {
            await subprocess_1.run('pod', ['install'], {
                ...opts,
                cwd: path_1.resolve(details.dir, 'ios'),
            });
        }
        catch (e) {
            process.stderr.write(`WARN: Could not install pods: ${(_f = (_e = e.message) !== null && _e !== void 0 ? _e : e.stack) !== null && _f !== void 0 ? _f : e}\n`);
        }
    }
    process.stdout.write('Initializing git...\n');
    try {
        await subprocess_1.run('git', ['init'], opts);
        await subprocess_1.run('git', ['checkout', '-b', 'main'], opts);
        await subprocess_1.run('git', ['add', '-A'], opts);
        await subprocess_1.run('git', ['commit', '-m', 'Initial commit', '--no-gpg-sign'], opts);
    }
    catch (e) {
        process.stderr.write(`WARN: Could not initialize git: ${(_h = (_g = e.message) !== null && _g !== void 0 ? _g : e.stack) !== null && _h !== void 0 ? _h : e}\n`);
    }
    const tada = cli_1.emoji('🎉', '*');
    process.stdout.write(`
${kleur_1.default.bold(`${tada} Capacitor plugin generated! ${tada}`)}

Next steps:
  - ${kleur_1.default.cyan(`cd ${details.dir}/`)}
  - Open ${kleur_1.default.bold('CONTRIBUTING.md')} to learn about the npm scripts
  - Continue following these docs for plugin development: ${kleur_1.default.bold('https://capacitorjs.com/docs/v3/plugins/workflow')}
  - Questions? Feel free to open a discussion: ${kleur_1.default.bold('https://github.com/ionic-team/capacitor/discussions')}
  - Learn more about the Capacitor Community: ${kleur_1.default.bold('https://github.com/capacitor-community/welcome')} 💖
`);
};
